@@ -1,4 +1,3 @@
-new.ctmm <- methods::setClass("ctmm", representation("list",info="list"))
 new.covm <- methods::setClass("covm", representation("matrix",par="numeric",isotropic="logical"))
 
 #######################################
@@ -236,13 +235,13 @@ pars.covm <- function(COVM)
 }
 
 # returns the canonical parameters of a tau vector
-pars.tauv <- function(tau)
+pars.tauv <- function(tau,tauc=tau)
 {
-  if(length(tau)==0)
-  { return(tau) }
-  else if(tau[1] < Inf)
-  { return(tau) }
-  else if(length(tau)==1)
+  if(length(tauc)==0)
+  { return(NULL) }
+  else if(tauc[1] < Inf)
+  { return(tau[1:length(tauc)]) }
+  else if(length(tauc)==1)
   { return(NULL) }
   else
   { return(tau[-1]) }
@@ -263,14 +262,16 @@ ctmm.prepare <- function(data,CTMM,REML=FALSE)
     if(CTMM$tau[1]==Inf)
     {
       range <- FALSE
-      CTMM$tau[1] <- log(2^(52/2))*(last(data$t)-data$t[1])
+      # aim for OU/OUF decay that is half way to machine epsilon
+      CTMM$tau[1] <- log(2^((.Machine$double.digits-1)/2))*(last(data$t)-data$t[1]) 
+      # CTMM$tau[1] <- (last(data$t)-data$t[1]) / (.Machine$double.eps)^(1/4)
       
       # diffusion -> variance
       if(!is.null(CTMM$sigma))
       { 
-        T <- (CTMM$tau[1]-if(K>1){CTMM$tau[2]}else{0})
-        CTMM$sigma <- CTMM$sigma*T
-        CTMM$sigma@par[1] <- CTMM$sigma@par[1]*T
+        TAU <- CTMM$tau[1]
+        CTMM$sigma <- CTMM$sigma*TAU
+        CTMM$sigma@par[1] <- CTMM$sigma@par[1]*TAU
       }
     }
     
@@ -300,6 +301,7 @@ ctmm.prepare <- function(data,CTMM,REML=FALSE)
   return(CTMM)
 }
 
+# undo the above
 ctmm.repair <- function(CTMM)
 {
   if(!CTMM$range)
@@ -307,16 +309,15 @@ ctmm.repair <- function(CTMM)
     K <- length(CTMM$tau)
     
     # variance -> diffusion
-    T <- (CTMM$tau[1]-if(K>1){CTMM$tau[2]}else{0})
-    CTMM$sigma <- CTMM$sigma/T
-    CTMM$sigma@par[1] <- CTMM$sigma@par[1]/T
-    
+    TAU <- CTMM$tau[1]
+    CTMM$sigma <- CTMM$sigma/TAU
+    CTMM$sigma@par[1] <- CTMM$sigma@par[1]/TAU
     CTMM$tau[1] <- Inf
     
-    # delete garbate estimates
-    CTMM$mu <- NULL
-    CTMM$COV.mu <- NULL
-    CTMM$DOF.mu <- NULL
+    ## delete garbate estimates
+    # CTMM$mu <- NULL
+    # CTMM$COV.mu <- NULL
+    # CTMM$DOF.mu <- NULL
   }
   
   # erase evaluated mean vector from ctmm.prepare
